/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */



/**
 * Canvas.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: Canvas.java,v 1.10 2000/05/09 22:38:33 will Exp $
 *
 */

package org.csiro.svgv.display;

import org.csiro.svgv.graphic.*;
import org.csiro.svgv.reader.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.print.*;
import javax.swing.*;
import java.util.*;
import java.awt.image.*;

import EDU.Washington.grad.gjb.cassowary.*;

public class Canvas extends JComponent implements Printable {

  /**
   * Rectangle2D.Doubles describing the view bounds and canvas bounds
   */

  Rectangle2D.Double theView = new Rectangle2D.Double(0, 0, 0, 0);
  Rectangle theCanvas = new Rectangle(0, 0, 0, 0);

  private double screenToWorldX;
  private double screenToWorldY;
  private double worldToScreenX;
  private double worldToScreenY;

  private boolean noDoubleBuffer = false;

  Stroke currentStroke = null;
  Paint currentLinePaint = null;
  Paint currentFillPaint = null;
  Font currentFont = null;

  SvgDocument m_svg_document;
  void setSvgDocument(SvgDocument svg_document)  {
    System.err.println("Setting the SvgDocument for the canvas");	
    m_svg_document = svg_document;  
  }
  
  /**
   * The offscreen image used to double buffer the display.
   */

  transient private java.awt.Image offscreen = null;


  public Canvas() {
    offscreen = null;
  }

  boolean transformStyles = true;
  
  /**
   * Get the value of transformStyles.
   * @return Value of transformStyles.
   */
  
  public boolean getTransformStyles() {
    return transformStyles;
  }
  
  /**
   * Set the value of transformStyles.
   * @param v  Value to assign to transformStyles.
   */
  
  public void setTransformStyles(boolean v) {
    this.transformStyles = v;
  }


  boolean flipped = false;

  /**
   * Get the value of flipped.
   * @return Value of flipped.
   */
  
  public boolean getFlipped() {
    return flipped;
  }
  
  /**
   * Set the value of flipped.
   * @param v  Value to assign to flipped.
   */
  
  public void setFlipped(boolean v) {
    this.flipped = v;
  }
  



  /**
   * Reshapes the canvas to the specified bounding box.
   *
   * @param r The bounding box
   */

  public void setBounds(Rectangle r) {
    super.setBounds(r);

    System.err.println("Will was off his rocker: setBounds(Rectangle)");
    if ((theCanvas.width == r.width) &&
        (theCanvas.height == r.height)) return;
    theCanvas.width = r.width;
    theCanvas.height = r.height;
    checkView();
  }

  /**
   * Reshapes the canvas to the specified bounding box.
   *
   * @param x the x coordinate
   * @param y the y coordinate
   * @param width the width of the component
   * @param height the height of the component
   */

  public void setBounds(int x, int y, int width, int height) {
    super.setBounds(x, y, width, height);
    if ((theCanvas.width == width) &&
        (theCanvas.height == height)) return;
    theCanvas.width = width;
    theCanvas.height = height;
    System.err.println("");
    System.err.println("setBounds(int, int, int, int)");    
    
    System.err.println("Resize to " +
                       new Integer(width).toString() +
                       " by " + new Integer(height).toString());    
    theView.width	= screenToWorldX * theCanvas.width;
    theView.height	= screenToWorldY * theCanvas.height;       
    checkView();
  }

  /**
   * Reshapes the canvas to the specified width and height.
   *
   * @param width the width of the component
   * @param height the height of the component
   */

  public void setSize(int width, int height){
    super.setSize(width, height);
    theCanvas.width = width;
    theCanvas.height = height;    
    System.err.println("Will was off his rocker: setSize(int, int)");
    checkView();
  }

  /**
   * Reshapes the canvas to the specified dimension.
   *
   * @param d The new dimension.
   */

  public void setSize(Dimension d) {
    super.setSize(d);        
    System.err.println("Will was off his rocker: setSize(Dimension)");    
    theCanvas.width = d.width;
    theCanvas.height = d.height;
    checkView();
  }

  /**
   * The minimum size of the canvas.
   *
   * @return The minimum size of the canvas.
   */
  public Dimension getMinimumSize() {
    return new Dimension(100, 100);
  }

  /**
   * The preferred size of the canvas.
   *
   * @return The preferred size of the canvas.
   */

  public Dimension getPreferredSize() {
    return new Dimension(100, 100);
  }


  /**
   * Return the double buffered image if there is one
   *
   * @return The image that is what this canvas is drawing
   */

  public java.awt.Image getBackgroundImage() {
    return offscreen;
  }


  /**
   * @param g The graphics object that will paint in this region.
   */

  public void paint(Graphics g) {

    Rectangle r = getBounds();
    int width = r.width;
    int height = r.height;

    if (width != 0 || height != 0) {
      if (!noDoubleBuffer) {
        if (offscreen == null) {
          offscreen = createImage(r.width, r.height);
        }

        // it might still be null
        if (offscreen != null) {
          Graphics og = offscreen.getGraphics();
          og.setClip(0,0,r.width, r.height);

	  doPaint(og);

          g.drawImage(offscreen, 0, 0, null);
          og.dispose();
        }
      } else {
	g.setClip(0,0,r.width, r.height);
	//super.paint(g);
	doPaint(g);
      }
    }
  }

  // the affine transform used to transform the user space into
  // screen space - have it here just in case the user only wants 
  // to transform geometry, not the styles like pen width etc.

  AffineTransform trans = new AffineTransform();

  public void doPaint(Graphics g) {

    Graphics2D g2 = (Graphics2D) g;
    if (!noDoubleBuffer) {
      RenderingHints hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
						RenderingHints.VALUE_ANTIALIAS_ON);
      g2.setRenderingHints(hints);
    }

    // color the background
    Rectangle r = getBounds();
    Rectangle2D.Double view = getView();

    g2.setColor(getBackground());
    g2.fillRect(0, 0, r.width, r.height);
    
    // create a transformation for the shapes that will be drawn
    trans = new AffineTransform();
    
    if (flipped) {
      trans.scale(1f, -1f);
      trans.translate(0, -1.0f * r.height);
    }

    trans.scale(r.width/view.width, r.height/view.height); // zoom
    trans.translate(-1.0f * view.x, -1.0f * view.y); // translate

    // set the default paint (default stroke should already be set)
    g2.setPaint(getForeground());
    currentStroke = g2.getStroke();
    
    Graphic graphic = getGraphic();
      
    if (transformStyles) {
      g2.transform(trans);
    }
    
    if (graphic != null) {
      drawGraphic(g2, graphic);
    }
  }


  private boolean isVisible(Graphic graphic) {
    Rectangle2D graphicMBR = graphic.getMBR();

    return graphicMBR == null || 
      graphicMBR.intersects(theView.x, theView.y, theView.width, theView.height);

  }


  public void drawGraphic(Graphics2D g, Graphic graphic) {

    if (graphic != null) {

      if (isVisible(graphic)) {
	
	currentStroke = graphic.getStroke();
	g.setStroke(currentStroke);
	
	currentLinePaint = graphic.getLinePaint();
	g.setPaint(currentLinePaint);

	currentFillPaint = graphic.getFillPaint();

	currentFont = graphic.getFont();
	g.setFont(currentFont);
	
	if (graphic instanceof Group) {
	  
	  ArrayList elements = ((Group)graphic).getElements();
	  int size = elements.size();
	  
	  for (int i = 0; i < size; i++) {
	    Graphic graphic2 = (Graphic) elements.get(i);
	    drawGraphic(g, graphic2);
	  }
	  
	} else if (graphic instanceof Path) {
	  
	  drawPath(g, (Path)graphic);
	  
	} else if (graphic instanceof Text) {
	  
	  drawText(g, (Text)graphic);

	} else if (graphic instanceof org.csiro.svgv.graphic.Image) {

	  drawImage(g, (org.csiro.svgv.graphic.Image)graphic);
	  
	} else {
	  System.err.println("Unknown graphic type: " + graphic);
	}
	
      }
    }
  }

  public void drawPath(Graphics2D g, Path path) {

    GeneralPath p = path.getGeneralPath();

    Shape transShape = p;

    if (!transformStyles) {
      transShape = trans.createTransformedShape(p);
    }

    if (currentFillPaint != null) {

      if (!transformStyles) {

        if (currentFillPaint instanceof GradientPaint) {
          // need to transform the points in the GradientPaint
          GradientPaint gradientPaint =  (GradientPaint)currentFillPaint;
          Point2D newPoint1 = trans.transform(gradientPaint.getPoint1(), null);
          Point2D newPoint2 = trans.transform(gradientPaint.getPoint2(), null);
          currentFillPaint = new GradientPaint(newPoint1, gradientPaint.getColor1(),
					       newPoint2, gradientPaint.getColor2(),
					       gradientPaint.isCyclic());

        } else if (currentFillPaint instanceof TexturePaint) {
          TexturePaint texturePaint = (TexturePaint)currentFillPaint;
          BufferedImage image = texturePaint.getImage();
          Rectangle2D rect = texturePaint.getAnchorRect();
          Point2D topLeft = trans.transform(new Point2D.Double(rect.getX(), rect.getY()), null);
          Point2D bottomRight = trans.transform(new Point2D.Double(rect.getX()+rect.getWidth(), 
                                                                   rect.getY()+rect.getHeight()), null);
          Rectangle2D transRect = new Rectangle2D.Double(topLeft.getX(), topLeft.getY(),
                                                         bottomRight.getX()-topLeft.getX(), 
                                                         bottomRight.getY()-topLeft.getY());
          currentFillPaint = new TexturePaint(image, transRect);

        }
      }
      g.setPaint(currentFillPaint);
      g.fill(transShape);
    }
    if (currentLinePaint != null && currentStroke != null && ((BasicStroke)currentStroke).getLineWidth() != 0) {
  
      if (!transformStyles) {

        if (currentLinePaint instanceof GradientPaint) {
          // need to transform the points in the GradientPaint
          GradientPaint gradientPaint =  (GradientPaint)currentLinePaint;
          Point2D newPoint1 = trans.transform(gradientPaint.getPoint1(), null);
          Point2D newPoint2 = trans.transform(gradientPaint.getPoint2(), null);
          currentLinePaint = new GradientPaint(newPoint1, gradientPaint.getColor1(),
                                               newPoint2, gradientPaint.getColor2(),
                                               gradientPaint.isCyclic());

        } else if (currentLinePaint instanceof TexturePaint) {
          TexturePaint texturePaint = (TexturePaint)currentLinePaint;
          BufferedImage image = texturePaint.getImage();
          Rectangle2D rect = texturePaint.getAnchorRect();
          Point2D topLeft = trans.transform(new Point2D.Double(rect.getX(), rect.getY()), null);
          Point2D bottomRight = trans.transform(new Point2D.Double(rect.getX()+rect.getWidth(), 
								   rect.getY()+rect.getHeight()), null);
          Rectangle2D transRect = new Rectangle2D.Double(topLeft.getX(), topLeft.getY(),
							 bottomRight.getX()-topLeft.getX(), 
							 bottomRight.getY()-topLeft.getY());
          currentLinePaint = new TexturePaint(image, transRect);
        }
      }
      g.setPaint(currentLinePaint);
      g.draw(transShape);
    }
  }


  public void drawText(Graphics2D g, Text text) {


    if (currentFont != null) {

      Shape p = text.getShape();
      
      Shape transShape = p;
      
      if (!transformStyles) {
	transShape = trans.createTransformedShape(transShape);
      }
      
      if (currentFillPaint == null) {
	g.fill(transShape);
      } else {
	if (!transformStyles) {
	  if (currentFillPaint instanceof GradientPaint) {
            // need to transform the points in the GradientPaint
	    GradientPaint gradientPaint =  (GradientPaint)currentFillPaint;
	    Point2D newPoint1 = trans.transform(gradientPaint.getPoint1(), null);
	    Point2D newPoint2 = trans.transform(gradientPaint.getPoint2(), null);
	    currentFillPaint = new GradientPaint(newPoint1, gradientPaint.getColor1(),
						 newPoint2, gradientPaint.getColor2(),
						 gradientPaint.isCyclic());
	  }
	}
	g.setPaint(currentFillPaint);
	g.fill(transShape);
	if (((BasicStroke)currentStroke).getLineWidth() != 0) {
	  if (!transformStyles) {
	    if (currentLinePaint instanceof GradientPaint) {
	      // need to transform the points in the GradientPaint
	      GradientPaint gradientPaint =  (GradientPaint)currentLinePaint;
	      Point2D newPoint1 = trans.transform(gradientPaint.getPoint1(), null);
	      Point2D newPoint2 = trans.transform(gradientPaint.getPoint2(), null);
	      currentLinePaint = new GradientPaint(newPoint1, gradientPaint.getColor1(),
						   newPoint2, gradientPaint.getColor2(),
						   gradientPaint.isCyclic());
	    }
	  }
	  g.setPaint(currentLinePaint);
	  g.draw(transShape);
	}
      }
    }
  }


  public void drawImage(Graphics2D g, org.csiro.svgv.graphic.Image image) {

    java.awt.Image realImage = image.getImage();

    AffineTransform at = null;
    if (!transformStyles) {
      at = new AffineTransform(trans);
    } else {
      at = new AffineTransform();
    }
    AffineTransform at2 = new AffineTransform(image.getTransform());
    
    int width = realImage.getWidth(null);
    int height = realImage.getHeight(null);

    double drawWidth = image.getWidth();
    double drawHeight = image.getHeight();

    at.translate(image.getX(), image.getY());
    at.concatenate(at2);
    at.scale(drawWidth / width, drawHeight / height);

    g.drawImage(realImage, at, null);
  }


  public void drawXORRectangle(Rectangle r) {

    Graphics g = getGraphics();
    g.setXORMode(getBackground());
    Rectangle s = new Rectangle(r);
    
    if (r.width < 0) {
      s.x = r.x + r.width;
      s.width = -1 * r.width;
    }
    
    if (r.height < 0) {
      s.y = r.y + r.height;
      s.height = -1 * r.height;
    }
    
    g.drawRect(s.x, s.y, s.width, s.height);
    
    g.dispose();
    g = null;
    
  }



  /**
   * Prints the spatial entities in the given print graphics object
   */

  public void print() {

    System.err.println("In Print");
    
    PrinterJob job = PrinterJob.getPrinterJob();
    job.setPrintable(this);
    
    if (job.printDialog()) {
      try { 
	job.print(); 
      } catch (Exception e1) {
	e1.printStackTrace();
      }
    }

  }

  public int print(Graphics g, PageFormat pf, int pageIndex) throws PrinterException {
    
    if (pageIndex > 0) {
      return Printable.NO_SUCH_PAGE;
    } else {
      Graphics2D g2 = (Graphics2D) g;
      g2.translate(pf.getImageableX(), pf.getImageableY());
      double scalex = pf.getImageableWidth() / (double) getWidth();
      double scaley = pf.getImageableHeight() / (double) getHeight();
      double scale = Math.min(scalex, scaley);
      g2.scale(scale, scale);
      System.err.println(pf.getImageableWidth() + " " + pf.getImageableHeight());
      System.err.println(pf.getWidth() + " " + pf.getHeight());
      System.err.println(getWidth() + " " + getHeight());

      //g2.scale(pf.getImageableWidth(), pf.getImageableHeight());

      System.err.println("printing " + pageIndex);
      
      noDoubleBuffer = true;
      Color oldBackground = getBackground();
      setBackground(Color.white);

      printAll(g2);
      
      setBackground(oldBackground);
      noDoubleBuffer = false;

      return Printable.PAGE_EXISTS;
    }
        
  }

  /**
   * Distributes the MouseEvent to all listeners. The MouseEvent is first
   * converted into a WorldMouseEvent which contains world coordinates.
   *
   * @param e The MouseEvent
   */

  public void processMouseEvent(MouseEvent e) {
    double worldX = getWorldXCoord(e.getX());
    double worldY;
    if (flipped) {
      worldY = getWorldYCoord(theCanvas.height - e.getY());
    } else {
      worldY = getWorldYCoord(e.getY());
    }
    super.processMouseEvent(new WorldMouseEvent(e, worldX, worldY));
  }

  /**
   * Distributes the MouseEvent to all mouse motion listeners. The MouseEvent is first
   * converted into a WorldMouseEvent which contains world coordinates.
   *
   * @param e The MouseEvent
   */

  public void processMouseMotionEvent(MouseEvent e) {
    double worldX = getWorldXCoord(e.getX());
    double worldY;
    if (flipped) {
      worldY = getWorldYCoord(theCanvas.height - e.getY());
    } else {
      worldY = getWorldYCoord(e.getY());
    }
    super.processMouseMotionEvent(new WorldMouseEvent(e, worldX, worldY));
  }

  /**
   * The actual thing to draw
   */

  Graphic graphic = null;


  /**
   * Sets the graphic to be drawn by this canvas
   */

  public void setGraphic(Graphic graphic) {
    this.graphic = graphic;
  }

  /**
   * Gets the graphic that is drawn by this canvas.
   *
   * @return The world drawn by this canvas
   */

  public Graphic getGraphic() {
    return graphic;
  }

  /**
   * Draws the objects in this canvas.
   */

  public void draw() {

    // schedule repaint

    // replaced repaint with a direct call to paint
    // for some reason repaint would call paint
    // before calling update - causing the display to flicker

    //paint(getGraphics());
    repaint();
  }

  /**
   * Override this function to make sure that the offscreen buffer
   * is cleared when the component is invalidated.
   */

  public void invalidate() {
    // make sure offscreen buffer is cleared when the component is invalidated.
    super.invalidate();
    offscreen = null;
  }

  /**
   * Override update to make sure that the component is not "blanked"
   * before paint is called.
   */

  public void update(Graphics g) {
    // implement update to not blank the window first
    // avoids flicker in double buffer mode
    paint(g);
  }

  /**
   * Checks that the view has positive width and height.
   * If you don't do this and someone sets the view width
   * and height to be negative, everything is backwards or
   * upside down or both!.
   * <p>
   * It also checks to make sure the aspect is the same as the
   * canvas aspect.
   * <p>
   * Also calls <code>calculateView</code>
   */

  private void checkView() {

    if (theView.width < 0) {
      theView.x = theView.x + theView.width;
      theView.width = theView.width * -1;
    }

    if (theView.height < 0) {
      theView.y = theView.y + theView.height;
      theView.height = theView.height * -1;
    }

    theView.x	= theCanvas.x;
    theView.y	= theCanvas.y;

    theView.width	= theCanvas.width;
    theView.height	= theCanvas.height;


    /*
      double minX = theView.x;
      double minY = theView.y;

      double zoomWidth = theView.width;
      double zoomHeight = theView.height;;

      //      // make sure we have not zoomed in past half the canvas size

      //      if (zoomWidth < theCanvas.width / 2.0) {
      //        double centerX = theView.x + theView.width / 2.0;
      //        zoomWidth = theCanvas.width / 2.0;
      //        minX = centerX - zoomWidth / 2.0;
      //      }

      //      if (zoomHeight < theCanvas.height / 2) {
      //        double centerY = theView.y + theView.height / 2.0;
      //        zoomHeight = theCanvas.height / 2.0;
      //        minY = centerY - zoomHeight / 2.0;
      //      }

      double zoomXratio = (double) theCanvas.width / (double) zoomWidth;
      double zoomYratio = (double) theCanvas.height / (double) zoomHeight;

      if (zoomXratio > zoomYratio) {

      // Y is more important

      theView.y = minY;
      theView.height = zoomHeight;

      theView.width = (double) (theCanvas.width * (theView.height / (double)theCanvas.height));
      theView.x = minX - (double) ((theView.width - zoomWidth) * 0.5);

      } else {

      // X is more important

      theView.x = minX;
      theView.width = zoomWidth;

      theView.height = (double) (theCanvas.height * (theView.width / (double)theCanvas.width));
      theView.y = minY - (double) ((theView.height - zoomHeight) * 0.5);

      }
    */
    calculateView();
  }

  public void updateViewportVariables()
  {
    if (m_svg_document != null)
      {
        System.err.println("Updating the viewport variables.");

        ClSimplexSolver simplex_solver = m_svg_document.m_simplex_solver;

		
        String viewport_width_variable_name = new String("viewportwidth");

	boolean using_viewport_width = m_svg_document.m_variable_name_object_hash.containsKey(viewport_width_variable_name);

        ClVariable viewport_width = null;

	boolean need_to_update_viewport_width = false;


        String viewport_height_variable_name = new String("viewportheight");

        boolean using_viewport_height = m_svg_document.m_variable_name_object_hash.containsKey(viewport_height_variable_name);

        ClVariable viewport_height = null;

	boolean need_to_update_viewport_height = false;


        try
        {
          long start_time = java.lang.System.currentTimeMillis();

          if (using_viewport_width)
          {
            viewport_width = (ClVariable) m_svg_document.m_variable_name_object_hash.get(viewport_width_variable_name);

            System.err.println("Viewport width was " + viewport_width.value() + " in the solver.");

	    if (viewport_width.value() != theView.width)
	    {
              System.err.println("Updating the viewport width to " + new Double(theView.width).toString());
					
              simplex_solver.addEditVar(viewport_width);

	      need_to_update_viewport_width = true;
	    }
	  }

	  if (using_viewport_height)
	  {
            viewport_height = (ClVariable) m_svg_document.m_variable_name_object_hash.get(viewport_height_variable_name);

            System.err.println("Viewport height was " + viewport_height.value() + " in the solver.");

	    if (viewport_height.value() != theView.height)
	    {	
              System.err.println("Updating the viewport height to " + new Double(theView.height).toString());

	      simplex_solver.addEditVar(viewport_height);

	      need_to_update_viewport_height = true;
	    }
	  }

	  simplex_solver.beginEdit();

	  if (need_to_update_viewport_width)
          {
	    simplex_solver.suggestValue(viewport_width, theView.width);
          }

	  if (need_to_update_viewport_height)
          {
            simplex_solver.suggestValue(viewport_height, theView.height);
          }

	  simplex_solver.resolve();
	  simplex_solver.endEdit();

          long end_time = java.lang.System.currentTimeMillis();
  
          long total_time = end_time - start_time;
    
          System.out.println("It took " + total_time + " milliseconds to update the viewport dimensions.");

        }
        catch (Exception exception)
        {
          System.err.println("Exception: " + exception.getMessage());
          exception.printStackTrace();
        };
	
        System.err.println("Getting the graphics again");
        Graphic graphic = m_svg_document.getSvgGraphics();
        setGraphic(graphic);
        m_svg_document.outputCnVarValues(System.err);
      }
  }
  /**
   * Calculate the world --> screen numbers so I don't have to do
   * the division for every point.
   */
  private void calculateView() {
    // do not recalc if theCanvas's size isn't yet initialized
    if (theCanvas.width == 0 ||
        theCanvas.height == 0)
      return;
    screenToWorldX = theView.width / (double)theCanvas.width;
    screenToWorldY = theView.height / (double)theCanvas.height;
    worldToScreenX = theCanvas.width / (double)theView.width;
    worldToScreenY = theCanvas.height / (double)theView.height;

    System.err.println("view.width    = " + new Double(theView.width ).toString());	
    System.err.println("view.height   = " + new Double(theView.height).toString());	
    System.err.println("canvas.width  = " + new Double(theCanvas.width).toString());	
    System.err.println("canvas.height = " + new Double(theCanvas.height).toString());
	
    updateViewportVariables();  }


  /**
   * Sets the current view to the given rectangle in world coordinates.
   * <p>
   * NOTE: The view may be scaled horizontally or vertically to maintain aspect ratio.
   *
   * @param view The new view
   */

  public void setView(Rectangle2D.Double view) {
    if (theView.equals(view)) return;

    theView = new Rectangle2D.Double(view.x, view.y, view.width, view.height);
    checkView();
    draw();
  }

  /**
   * Sets the current view to the given rectangle in world coordinates.
   * <p>
   * NOTE: The view may be scaled horizontally or vertically to maintain aspect ratio.
   *
   * @param x The x origin of the new view
   * @param y The y origin of the new view
   * @param width The width of the new view
   * @param height The height of the new view
   */

  public void setView(double x, double y, double width, double height) {
    if ((theView.x == x) &&
        (theView.y == y) &&
        (theView.width == width) &&
        (theView.height == height)) return;

    theView.x = x;
    theView.y = y;
    theView.width = width;
    theView.height = height;
    checkView();

    draw();
  }

  /**
   * Returns the current view.
   *
   * @return The Rectangle that describes the current view.
   */

  public Rectangle2D.Double getView() {
    return new Rectangle2D.Double(theView.x, theView.y, theView.width, theView.height);
  }

  /**
   * Returns the x value of the world coordinate that corresponds to the given canvas
   * x value.
   *
   * @param screenX The x value of the canvas coordinate
   * @return The world x coordinate value
   */

  public double getWorldXCoord(int screenX) {
    return (screenX * screenToWorldX) + theView.x;
  }

  /**
   * Returns the y value of the world coordinate that corresponds to the given canvas
   * y value.
   *
   * @param screenY The y value of the canvas coordinate
   * @return The world y coordinate value
   */

  public double getWorldYCoord(int screenY) {
    return (screenY * screenToWorldY) + theView.y;
  }

  /**
   * Returns the x value of the canvas coordinate that corresponds to the given world
   * x value.
   *
   * @param worldX The x value of the world coordinate
   * @return The canvas x coordinate value
   */

  public int getScreenXCoord(double worldX) {
    return (int) ((worldX - theView.x) * worldToScreenX);
  }

  /**
   * Returns the y value of the canvas coordinate that corresponds to the given world
   * y value.
   *
   * @param worldY The y value of the world coordinate
   * @return The canvas y coordinate value
   */

  public int getScreenYCoord(double worldY) {
    return (int) ((worldY - theView.y) * worldToScreenY);
  }

  /**
   * Pans the current view by the given relative values.
   *
   * @param x The amount to pan in the horizontal direction
   * @param y The amount to pan in the vertical direction
   */

  public void panRelative(double x, double y) {
    theView.x += x;
    theView.y += y;
    calculateView();
  }

  /**
   * Pans the current view to the given absolute coordinates.
   *
   * @param x The new x origin for the view
   * @param y The new y origin for the view
   */

  public void panAbsolute(double x, double y) {
    theView.x = x;
    theView.y = y;
    calculateView();
  }

  /**
   * Centers the view around the given coordinates.
   *
   * @param x The x position of the new center
   * @param y The y position of the new center
   */

  public void center(double x, double y) {

    theView.x = x - (theView.width / 2.0);
    theView.y = y - (theView.height / 2.0);
    calculateView();

  }

  /**
   * Fits all the entities to the window.  If the worldView has been set, we
   * zoom to that; otherwise we zoom to the MBR of the spatial world
   */

  public void fitToWindow() {
    zoomAll();
  }

  /**
   * Zooms to fill all entities in the view. This relies on the entity manager
   * in canvas giving a view rectangle.
   */

  public void zoomAll() {

    boolean want_this_functionality = false;

    if (want_this_functionality)
      {

        if (graphic != null && graphic.getMBR() != null) {

          Rectangle2D r = graphic.getMBR();
          Rectangle2D.Double newView = new Rectangle2D.Double(r.getX(), r.getY(), r.getWidth(), r.getHeight());

          newView.x = newView.x - (0.05 * newView.width);
          newView.width = (1.1 * newView.width);
          newView.y = newView.y - (0.05 * newView.height);
          newView.height = (1.1 * newView.height);

          setView(newView);

        } else {
          System.err.println("Can't zoom: don't know how big the world is.");
        }
      }
  }


  /**
   * Zooms the view by provided scaling factor. The center point of the old
   * view will be the center point of the new view. To zoom about a specific point
   * calculate the new view rectangle and use setView.
   * <p>
   * To zoom in on a view use a ratio less than one. To zoom out on a view, use a ratio
   * greater than one.
   *
   * @param ratio The zoom ratio
   */

  public void zoom(double ratio) {
    Rectangle2D.Double newView = new Rectangle2D.Double();

    double midX = theView.x + (theView.width / 2.0);
    double midY = theView.y + (theView.height / 2.0);

    newView.width = (theView.width * ratio);
    newView.height = (theView.height * ratio);
    newView.x = midX - (newView.width / 2.0);
    newView.y = midY - (newView.height / 2.0);

    setView(newView);
  }

}
