/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * MouseHandler.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: MouseHandler.java,v 1.1.1.1 1999/12/11 18:51:27 gjb Exp $
 *
 */

package org.csiro.svgv.display;

import java.awt.event.*;

public abstract class MouseHandler
	implements MouseListener, MouseMotionListener {

  /**
   * The Canvas that this mouse handler uses
   */
  Canvas canvas;

  /**
   * Constructs a Mouse Handler object with the given canvas. The
   * handler will perform some function on the canvas when it
   * is given mouse events.
   *
   * @param canvas The canvas this mouse handler is handling.
   */

  public MouseHandler(Canvas canvas) {
    setCanvas(canvas);
  }


  /**
   * Sets the Canvas object.
   *
   * @param canvas The new Canvas object.
   */
  public void setCanvas(Canvas canvas) {
    this.canvas = canvas;
  }


  /**
   * Gets the Canvas object.
   *
   * @return The Canvas that this handler is associated with.
   */
  public Canvas getCanvas() {
    return canvas;
  }

  /**
   * Invoked when the mouse has been clicked in the Canvas canvas.
   */

  public void mouseClicked(MouseEvent e) {

  }

  /**
   * Invoked when the mouse has been pressed in the Canvas canvas.
   */

  public void mousePressed(MouseEvent e) {

  }

  /**
   * Invoked when the mouse has been released in the Canvas canvas.
   */

  public void mouseReleased(MouseEvent e) {

  }

  /**
   * Invoked when the mouse has entered the Canvas canvas.
   */

  public void mouseEntered(MouseEvent e) {

  }

  /**
   * Invoked when the mouse has exited the Canvas canvas.
   *
   * This default implementation clears the status
   */

  public void mouseExited(MouseEvent e) {
  }

  /**
   * Invoked when a mouse button is pressed within the Canvas canvas and then
   * dragged.
   */

  public void mouseDragged(MouseEvent e) {

  }

  /**
   * Invoked when the mouse button has been moved in the Canvas canvas,
   * when there are no buttons pressed.
   *
   * This default implementation shows the current world coordinates as the
   * status
   */

  public void mouseMoved ( MouseEvent e ) {
  }
}
