/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * PanMouseHandler.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: PanMouseHandler.java,v 1.1.1.1 1999/12/11 18:51:27 gjb Exp $
 *
 */

package org.csiro.svgv.display;

import java.awt.event.*;

/**
 * PanMouseHandler is a mouse handler that will pan the display as the user
 * drags the mouse around the canvas.
 */

public class PanMouseHandler extends MouseHandler {

  /**
   * The original values to calculate pan.
   */

  double pressedX;
  double pressedY;

  /**
   * Constructs a PanMouseHandler with the given Canvas object.
   *
   * @param canvas The Canvas object.
   */
   
  public PanMouseHandler(Canvas canvas) {
    super(canvas);
  }

  /**
   * Invoked when the mouse has been pressed in the Canvas canvas.
   */

  public void mousePressed(MouseEvent e) {
    if (!e.isMetaDown()) {
      pressedX = ((WorldMouseEvent)e).getWorldX();
      pressedY = ((WorldMouseEvent)e).getWorldY();
    }
  }

  /**
   * Invoked when a mouse button is pressed within the Canvas canvas and then 
   * dragged.  
   */

  public void mouseDragged(MouseEvent e) {
    if (!e.isMetaDown()) {

      double deltaX = pressedX - ((WorldMouseEvent)e).getWorldX();
      double deltaY = pressedY - ((WorldMouseEvent)e).getWorldY();
      
      if (deltaX != 0 || deltaY != 0) {
	getCanvas().panRelative(deltaX, deltaY);
	getCanvas().draw();
      }
    }
  }
}
