/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * Viewer.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: Viewer.java,v 1.12 2000/05/09 20:34:48 will Exp $
 *
 */

package org.csiro.svgv.display;

import org.csiro.svgv.graphic.*;
import org.csiro.svgv.reader.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;

import EDU.Washington.grad.gjb.cassowary.*;

public class Viewer extends JApplet {
  
  public boolean invokedStandalone = false;

  // the svg canvas

  public org.csiro.svgv.display.Canvas canvas;

  // swing components that make up the display

  JPanel contentPanel = new JPanel();
  JPanel toolbar = new JPanel();
  JPopupMenu popupMenu = new JPopupMenu();

  // remembers the last directory that the user selected
  // a file from

  String currentDirectory = null;

  // if this is run as an applet then the frame will
  // be set by the main() method and the viewer can
  // manipulate the frame title

  JFrame frame = null;

  int width;
  int height;
  String title = "SVG Viewer Version 0.71";
  public String svgfile = null;

  // parameter to draw the canvas flipped
  // (useful for maps with bottom-up coordinates)

  boolean flipped = false;

  // event handlers

  PanMouseHandler panMouseHandler;
  ZoomInMouseHandler zoomInMouseHandler;
  ZoomOutMouseHandler zoomOutMouseHandler;
  ZoomRectMouseHandler zoomRectMouseHandler;
  MouseHandler currentMouseHandler;
  PopupMenuHandler popupMenuHandler;


  public Viewer() {
    super();
    m_current_time = 0.0;
    m_fixed_time_rate = 0.2;
  }

  boolean showToolbar = true;
  
  /**
   * Get the value of showToolbar.
   * @return Value of showToolbar.
   */
  
  public boolean getShowToolbar() {
    return showToolbar;
  }
  
  /**
   * Set the value of showToolbar.
   * @param v  Value to assign to showToolbar.
   */
  
  public void setShowToolbar(boolean v) {
    if (v && !showToolbar) {
      contentPanel.add(toolbar, "North");
      contentPanel.revalidate();
      if (canvas != null) {
	canvas.draw();
      }
    } else if (!v && showToolbar) {
      contentPanel.remove(toolbar);
      contentPanel.revalidate();
      if (canvas != null) {
	canvas.draw();
      }
    }

    this.showToolbar = v;

  }
  
  boolean showMenu = true;
  
  /**
   * Get the value of showMenu.
   * @return Value of showMenu.
   */
  
  public boolean getShowMenu() {
    return showMenu;
  }
  
  /**
   * Set the value of showMenu.
   * @param v  Value to assign to showMenu.
   */
  
  public void setShowMenu(boolean v) {
    this.showMenu = v;
  }

  Dimension buttonDimension = new Dimension(39, 39);
  Insets zeroInsets = new Insets(0, 0, 0, 0);

  private java.net.URL getImage(String name) {
    return getClass().getResource(name);
  }

  protected JButton makeToolbarButton(String tip, String imagename) {
    ImageIcon imageIcon = new ImageIcon(getImage(imagename));
    JButton b1;
    if (imageIcon.getIconWidth() < 1 || imageIcon.getIconHeight() < 1) {
      b1 = new JButton(tip);
    } else {
      b1 = new JButton(imageIcon);
    }
    b1.setMinimumSize(buttonDimension);
    b1.setMaximumSize(buttonDimension);
    b1.setPreferredSize(buttonDimension);
    b1.setMargin(zeroInsets);
    b1.setToolTipText(tip);
    toolbar.add(b1);
    return b1;
  }

  protected JMenuItem makeMenuItem(String text, boolean isCheck) {
    JMenuItem menuItem;

    if (isCheck) {
      menuItem = new JCheckBoxMenuItem(text, false);
    } else {
      menuItem = new JMenuItem(text);
    }
    popupMenu.add(menuItem);

    return menuItem;
  }

  public void setFlipped(boolean v) {
    flipped = v;
  }

  protected void initParams() {

    if (!invokedStandalone) {

      if (getParameter("svgfile") != null) {
	svgfile = getParameter("svgfile");
	System.out.println("setting svgfile to " + svgfile);
      }
      
      if (getParameter("showtoolbar") != null) {
	showToolbar = Boolean.valueOf(getParameter("showtoolbar")).booleanValue();
	System.out.println("setting showtoolbar to " + showToolbar);
      }
      
      if (getParameter("showmenu") != null) {
	showMenu = Boolean.valueOf(getParameter("showmenu")).booleanValue();
	System.out.println("setting showMenu to " + showMenu);
      }
      
      if (getParameter("flipped") != null) {
	flipped = Boolean.valueOf(getParameter("flipped")).booleanValue();
	System.out.println("setting flipped to " + flipped);
      }
      
      if (getParameter("bgcolor") != null) {
	try {
	  Color color = Color.decode(getParameter("bgcolor"));
	  canvas.setBackground(color);
	} catch (NumberFormatException e) {
	  
	}

	if (getParameter("dt") != null)
	{
		m_fixed_time_rate = Double.valueOf(getParameter("dt")).doubleValue();
	}

      }

    }
  }

  public void start() {
    System.out.println("In start");
    canvas.fitToWindow();
    canvas.draw();
  }

  public void init() {

    System.out.println("In initialise");

    contentPanel.setLayout(new BorderLayout());
    getContentPane().add(contentPanel, "Center");
    
    toolbar.setBorder(BorderFactory.createEtchedBorder());
    toolbar.setLayout(new FlowLayout(FlowLayout.LEFT));

    JButton b1 = makeToolbarButton("Open a file", "/images/open.gif");
    JMenuItem m1 = makeMenuItem("Open..", false);
    OpenListener openListener = new OpenListener();
    openListener.viewer = this;
    b1.addActionListener(openListener);
    m1.addActionListener(openListener);

    JButton b2 = makeToolbarButton("Fit to window", "/images/zoom-all.gif");
    JMenuItem m2 = makeMenuItem("Fit to window", false);
    FitToWindowListener fitToWindowListener = new FitToWindowListener();
    b2.addActionListener(fitToWindowListener);
    m2.addActionListener(fitToWindowListener);

    // add a separator to the menu
    popupMenu.addSeparator();

    JButton b3 = makeToolbarButton("Zoom In", "/images/zoom-in.gif");
    JMenuItem m3 = makeMenuItem("Zoom In", true);
    ZoomInListener zoomInListener = new ZoomInListener();
    b3.addActionListener(zoomInListener);
    m3.addActionListener(zoomInListener);

    JButton b4 = makeToolbarButton("Zoom Out", "/images/zoom-out.gif");
    JMenuItem m4 = makeMenuItem("Zoom Out", true);
    ZoomOutListener zoomOutListener = new ZoomOutListener();
    b4.addActionListener(zoomOutListener);
    m4.addActionListener(zoomOutListener);

    JButton b5 = makeToolbarButton("Zoom to Rectangle", "/images/zoom-rect.gif");
    JMenuItem m5 = makeMenuItem("Zoom Rectangle", true);
    ZoomRectListener zoomRectListener = new ZoomRectListener();
    b5.addActionListener(zoomRectListener);
    m5.addActionListener(zoomRectListener);

    JButton b6 = makeToolbarButton("Pan", "/images/pan.gif");
    JMenuItem m6 = makeMenuItem("Pan", true);
    PanListener panListener = new PanListener();
    b6.addActionListener(panListener);
    m6.addActionListener(panListener);

    // add a separator to the menu
    popupMenu.addSeparator();

    JButton b7 = makeToolbarButton("Print", "/images/print.gif");
    JMenuItem m7 = makeMenuItem("Print...", false);
    PrintListener printListener = new PrintListener();
    b7.addActionListener(printListener);
    m7.addActionListener(printListener);

    JButton b8 = makeToolbarButton("Reset Time", "/images/clock.gif");
    JMenuItem m8 = makeMenuItem("Reset Time", false);
    ResetTimeListener reset_time_listener = new ResetTimeListener();
    b8.addActionListener(reset_time_listener);
    m8.addActionListener(reset_time_listener);

    canvas = new org.csiro.svgv.display.Canvas();
    canvas.setBackground(Color.white);
    contentPanel.add(canvas, "Center");

    // initialise the parameters

    initParams();


    // load and view the file if it exists
    if (this.svgfile != null) {
      loadDocument(this.svgfile);
      canvas.setSvgDocument(doc);
//	  canvas.updateViewportVariables();
	  canvas.zoomAll();
    }

    canvas.setFlipped(flipped);

    zoomInMouseHandler = new ZoomInMouseHandler(canvas);
    zoomOutMouseHandler = new ZoomOutMouseHandler(canvas);
    zoomRectMouseHandler = new ZoomRectMouseHandler(canvas);
    panMouseHandler = new PanMouseHandler(canvas);

    popupMenuHandler = new PopupMenuHandler(canvas);
    canvas.addMouseListener(popupMenuHandler);
    canvas.addMouseMotionListener(popupMenuHandler);

    if (showToolbar) {
      contentPanel.add(toolbar, "North");
    }

  }

  private SvgDocument doc;

  private double m_current_time;

  private double m_fixed_time_rate;

  public void setFixedTimeRate(double fixed_time_rate)
  {
    m_fixed_time_rate = fixed_time_rate;
  }

  public void loadDocument(String docName) {

    this.svgfile = docName;

    // erase the current graphics

    canvas.setGraphic(null);

    // create a new document and load the svg
    
    doc = new SvgDocument();

    if (doc.readSvgUrl(docName)) {

      Graphic graphic = doc.getSvgGraphics();
      canvas.setGraphic(graphic);
      
      title = "SVG Viewer";
      if (doc.getSvgTitle() != null) {
	title += " : " + doc.getSvgTitle();
      } 

      // set the frame title if there is a frame
      if (frame != null) {
	frame.setTitle(title);
      }
      
    } else {
      
      // couldn't read the svg - popup an error message
      
      JOptionPane.showMessageDialog(this, "Could not load SVG :\n" + docName, 
				    "Load Error", JOptionPane.INFORMATION_MESSAGE);

    }

	try
	{
		for (Enumeration e = doc.m_variable_name_object_hash.elements() ; e.hasMoreElements() ;)
		{
			doc.m_simplex_solver.addStay((ClVariable) e.nextElement());
		}
	}
	catch (Exception exception)
	{
		exception.printStackTrace();
	}
		
 
    // now redraw the canvas
    canvas.fitToWindow();
    canvas.draw();

    java.lang.Thread thread = new java.lang.Thread()
      {
        private boolean updateVariableIfUsed(String variable_name, double variable_value)
        {
          boolean updated_variable = false;

          if (doc != null)			       
          {        
            if (doc.m_variable_name_object_hash.containsKey(variable_name))
            {
              System.err.println("Updating " + variable_name + " to " + variable_value);
                        
              ClVariable variable = (ClVariable) doc.m_variable_name_object_hash.get(variable_name);
                        
              System.err.println("Current Value was " + variable.value() + " in the solver.");
                           
              try
              {
                long start_time = java.lang.System.currentTimeMillis();

                doc.m_simplex_solver.addEditVar(variable);
                doc.m_simplex_solver.beginEdit();
                doc.m_simplex_solver.suggestValue(variable, variable_value);
                doc.m_simplex_solver.resolve();
                doc.m_simplex_solver.endEdit();

                long end_time = java.lang.System.currentTimeMillis();
  
                long total_time = end_time - start_time;
    
                System.out.println("It took " + total_time + " milliseconds to update " + variable_name);
              }
              catch (Exception exception)
              {
                System.out.println("Exception: " + exception.getMessage());
                exception.printStackTrace();
              }

              updated_variable = true;
            }
          } // doc != null
          else
          {
            System.err.println("Not updating " + variable_name);

            updated_variable = false;
          }

          return updated_variable;
        }

        public void run()
        {
          while(true)
            {
              m_current_time += m_fixed_time_rate;

              double current_time_squared = m_current_time * m_current_time;

              boolean current_time_used = updateVariableIfUsed("currenttime", m_current_time);

              boolean current_time_squared_used = updateVariableIfUsed("currenttimesquared", current_time_squared);

              boolean need_to_redraw = current_time_used || current_time_squared_used;

              if (need_to_redraw)
              {
                Graphic graphic = doc.getSvgGraphics();
                canvas.setGraphic(graphic);

                canvas.zoomAll();
                canvas.draw();
              }

              try
              {
                sleep(new Double(m_fixed_time_rate * 1000).longValue());
              }
              catch (Exception exception)
              {
                System.err.println("Exception: " + exception.getMessage());
                exception.printStackTrace();
              }
            } // while true
        }
      };

    thread.start();
  }


  public static void main (String args[]) {

    // check the command line

    if (args.length > 4) {
      System.out.println("Usage: java Viewer [-/+toolbar] [-/+menu] [-/+flipped] [svg-url]");
      System.exit(1);
    }
    
    boolean showToolbar = true;
    boolean showMenu = true;
    boolean flipped = false;
    String svgurl = null;
    
    double delay = -1.0;

    for (int i=0; i < args.length; i++) {
      if (args[i].equals("+toolbar")) {
	showToolbar = false;
      } else if (args[i].equals("+menu")) {
	showMenu = false;
      } else if (args[i].equals("-flipped")) {
	flipped = true;
      } else if (args[i].equals("-delay")) {
        delay = Double.valueOf(args[++i]).doubleValue();
      } else {
	// must be a file
	svgurl = args[i];
      }
    }

    // create the applet with the svg-url as argument

    Viewer viewer = new Viewer();

    viewer.invokedStandalone = true;
    if (svgurl != null) {
      viewer.svgfile = svgurl;
    }
    viewer.setShowToolbar(showToolbar);
    viewer.setShowMenu(showMenu);
    viewer.setFlipped(flipped);
    if (delay > 0) {
      viewer.setFixedTimeRate(delay);
    }

    JFrame f = new JFrame("SVG Viewer");
    f.setBounds(0, 0, 600, 600);
    f.getContentPane().add(viewer, "Center");
    viewer.frame = f;
    viewer.init();

    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });

    f.show();

    // zoom to all entities and draw canvas

    viewer.canvas.fitToWindow();
    viewer.canvas.draw();
  }


  // Inner classes for event handling 


  public class OpenListener implements ActionListener {
    
    public Viewer viewer = null;

    public void actionPerformed(ActionEvent e) {

      Object[] options = {"Open", "Cancel", "Browse for File"};

      JOptionPane pane = new JOptionPane("Enter the filename or URL to open\n(or browse for a file)", 
					 JOptionPane.QUESTION_MESSAGE,
					 JOptionPane.OK_CANCEL_OPTION,
					 null, options, options[0]);
      pane.setWantsInput(true);
      if (svgfile != null) {
	pane.setInitialSelectionValue(svgfile);
      }
      pane.selectInitialValue();

      JDialog dialog = pane.createDialog(viewer, title);
      dialog.show();
      String selectedValue = (String) pane.getValue();

      if (selectedValue == null) {
	// user closed the dialog
  	return;
      }

      if (selectedValue == JOptionPane.UNINITIALIZED_VALUE || 
	  options[0].equals(selectedValue)) {
	// user entered URL and selected open
	String inputValue = (String) pane.getInputValue();
	loadDocument(inputValue);
    canvas.setSvgDocument(doc);
	canvas.updateViewportVariables();
	canvas.zoomAll();
      } else if (options[1].equals(selectedValue)) {
	// user selected cancel
	return;
      } else {
	// user wants to open a file
	JFileChooser chooser;
	if (currentDirectory == null)
	  chooser = new JFileChooser(System.getProperty("user.dir"));
	else
	  chooser = new JFileChooser(currentDirectory);
	int retval = chooser.showOpenDialog(canvas);
	if (retval == JFileChooser.APPROVE_OPTION) {
	  File f = chooser.getSelectedFile();
	  currentDirectory = f.getPath();
	  System.out.println(f.getAbsolutePath());
	  loadDocument(f.getAbsolutePath());
      canvas.setSvgDocument(doc);
	  canvas.updateViewportVariables();
	  canvas.zoomAll();
	}
      }
    }
  }

	public class ResetTimeListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			System.out.println("Reseting the current time now.");

			m_current_time = 0.0;
		}
	}

  public class FitToWindowListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {

	  Graphic graphic = doc.getSvgGraphics();
      canvas.setGraphic(graphic);

      canvas.zoomAll();
      canvas.draw();
    }
  }

  public class ZoomInListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      canvas.removeMouseListener(currentMouseHandler);
      canvas.removeMouseMotionListener(currentMouseHandler);
      currentMouseHandler = zoomInMouseHandler;
      canvas.addMouseListener(currentMouseHandler);
      canvas.addMouseMotionListener(currentMouseHandler);
    }
  }

  public class ZoomOutListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      canvas.removeMouseListener(currentMouseHandler);
      canvas.removeMouseMotionListener(currentMouseHandler);
      currentMouseHandler = zoomOutMouseHandler;
      canvas.addMouseListener(currentMouseHandler);
      canvas.addMouseMotionListener(currentMouseHandler);
    }
  }

  public class ZoomRectListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      canvas.removeMouseListener(currentMouseHandler);
      canvas.removeMouseMotionListener(currentMouseHandler);
      currentMouseHandler = zoomRectMouseHandler;
      canvas.addMouseListener(currentMouseHandler);
      canvas.addMouseMotionListener(currentMouseHandler);
    }
  }

  public class PanListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      canvas.removeMouseListener(currentMouseHandler);
      canvas.removeMouseMotionListener(currentMouseHandler);
      currentMouseHandler = panMouseHandler;
      canvas.addMouseListener(currentMouseHandler);
      canvas.addMouseMotionListener(currentMouseHandler);
    }
  }

  public class PrintListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      canvas.print();
    }
  }
  

  public class PopupMenuHandler extends MouseHandler {

    public PopupMenuHandler(org.csiro.svgv.display.Canvas canvas) {
      super(canvas);
    }

    public void mousePressed(MouseEvent e) {
      if (e.isMetaDown()) {
	if (showMenu) {
	  MenuElement[] elements = popupMenu.getSubElements();
	  for (int i=0; i < elements.length; i++) {
	    JMenuItem item = (JMenuItem) elements[i];

	    if (item instanceof JCheckBoxMenuItem) {
	      if (currentMouseHandler == zoomInMouseHandler && 
		  item.getText().equals("Zoom In")) {
		((JCheckBoxMenuItem)item).setState(true);
	      } else if (currentMouseHandler == zoomOutMouseHandler && 
			 item.getText().equals("Zoom Out")) {
		((JCheckBoxMenuItem)item).setState(true);
	      } else if (currentMouseHandler == zoomRectMouseHandler && 
			 item.getText().equals("Zoom Rectangle")) {
		((JCheckBoxMenuItem)item).setState(true);
	      } else if (currentMouseHandler == panMouseHandler && 
			 item.getText().equals("Pan")) {
		((JCheckBoxMenuItem)item).setState(true);
	      } else {
		((JCheckBoxMenuItem)item).setState(false);
	      }
	    }
	  }
	  popupMenu.show(this.canvas, e.getX(), e.getY());
	}
      }
    }

  }


}
