/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * ZoomRectMouseHandler.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: ZoomRectMouseHandler.java,v 1.1.1.1 1999/12/11 18:51:27 gjb Exp $
 *
 */

package org.csiro.svgv.display;

import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.awt.Rectangle;

/**
 * ZoomRectMouseHandler is a mouse handler that will ZoomRect the display as the user
 * drags the mouse around the canvas.
 */

public class ZoomRectMouseHandler extends MouseHandler {

  /**
   * The canvas dimensions.
   */

  Rectangle canvasDimensions;

  /**
   * Where the mouse was pressed.
   */

  int pressedX;
  int pressedY;

  /**
   * The previous rectangle that was drawn on the screen
   */

  Rectangle previous = null;

  /**
   * Constructs a ZoomRectMouseHandler with the given Canvas object.
   *
   * @param canvas The Canvas object.
   */
   
  public ZoomRectMouseHandler(Canvas canvas) {
    super(canvas);
  }

  /**
   * Invoked when the mouse has been pressed in the Canvas canvas.
   */

  public void mousePressed(MouseEvent e) {
    
    if (!e.isMetaDown()) {
    
      pressedX = e.getX();
      pressedY = e.getY();
      
      canvasDimensions = getCanvas().getBounds();
      previous = new Rectangle(pressedX, pressedY, canvasDimensions.width / 10, canvasDimensions.height / 10);
      getCanvas().drawXORRectangle(previous);
    }
  }

  /**
   * Invoked when the mouse has been released in the canvas.
   */
  
  public void mouseReleased(MouseEvent e) {

    if (!e.isMetaDown()) {

      Rectangle r = new Rectangle(previous);
    
      if (r.width < 0) {
	r.x = r.x + r.width;
	r.width = -1 * r.width;
      }
      if (r.height < 0) {
	r.y = r.y + r.height;
	r.height = -1 * r.height;
      }
    
      Rectangle2D.Double newView = new Rectangle2D.Double(0d, 0d, 0d, 0d);
      Canvas c = getCanvas();


      newView.x = c.getWorldXCoord(r.x);
      newView.width = c.getWorldXCoord(r.x + r.width) - newView.x;

      if (c.getFlipped()) {
	newView.y = c.getWorldYCoord(canvasDimensions.height - r.y);
	newView.height = c.getWorldYCoord(canvasDimensions.height - r.y - r.height) - newView.y;
      } else {
	newView.y = c.getWorldYCoord(r.y + r.height);
	newView.height = c.getWorldYCoord(r.y) - newView.y;
      }    

      c.setView(newView);
      c.draw();
    
      previous = null;
    }  

  }

  /**
   * Invoked when a mouse button is pressed within the canvas and then 
   * dragged.  
   */

  public void mouseDragged(MouseEvent e) {

    if (!e.isMetaDown()) {

      // first remove the last rectangle
      if (previous != null) {
	getCanvas().drawXORRectangle(previous);
      
	int newWidth = e.getX() - pressedX;
	int newHeight = e.getY() - pressedY;
      
	if (newWidth > 0) {
	  previous.x = pressedX;
	  if (newWidth < canvasDimensions.width / 10) newWidth = canvasDimensions.width / 10;
	} else {
	  if ((-1 * newWidth) < canvasDimensions.width / 10) newWidth = canvasDimensions.width / -10;
	}
      
	if (newHeight > 0) {
	  if (newHeight < canvasDimensions.height / 10) newHeight = canvasDimensions.height / 10;
	} else {
	  if ((-1 * newHeight) < canvasDimensions.height / 10) newHeight = canvasDimensions.height / -10;
	}
      
	previous.width = newWidth;
	previous.height = newHeight;
      
	// now draw the new rectangle
	getCanvas().drawXORRectangle(previous);
      }
    }
  }

}
