/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */



/**
 * Graphic.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: Graphic.java,v 1.1.1.1 1999/12/11 18:51:27 gjb Exp $
 *
 */

package org.csiro.svgv.graphic;

import java.awt.Paint;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.Font;


public abstract class Graphic {

  public Graphic() {
    setId(null);
  }


  public Graphic(String id) {
    setId(id);
  }
  
  
  String id = null;
  
  public boolean hasId() {
    return (id != null);
  }

  /**
   * Get the value of id.
   * @return Value of id.
   */
  public String getId() {
    return id;
  }
  
  /**
   * Set the value of id.
   * @param v  Value to assign to id.
   */
  public void setId(String v) {
    this.id = v;
  }

  Stroke stroke = null;
  
  /**
   * Get the value of stroke.
   * @return Value of stroke.
   */
  public Stroke getStroke() {
    return stroke;
  }
  
  /**
   * Set the value of stroke.
   * @param v  Value to assign to stroke.
   */
  public void setStroke(Stroke v) {
    this.stroke = v;
  }

  
  Paint linePaint = null;
  
  /**
   * Get the value of linePaint.
   * @return Value of linePaint.
   */
  public Paint getLinePaint() {
    return linePaint;
  }
  
  /**
   * Set the value of linePaint.
   * @param v  Value to assign to linePaint.
   */
  public void setLinePaint(Paint v) {
    this.linePaint = v;
  }
  
 
  Paint fillPaint = null;
  
  /**
   * Get the value of fillPaint.
   * @return Value of fillPaint.
   */
  public Paint getFillPaint() {
    return fillPaint;
  }
  
  /**
   * Set the value of fillPaint.
   * @param v  Value to assign to fillPaint.
   */
  public void setFillPaint(Paint v) {
    this.fillPaint = v;
  }

  Font font = null;
  
  /**
   * Get the value of font.
   * @return Value of font.
   */
  
  public Font getFont() {
    return font;
  }
  
  /**
   * Set the value of font.
   * @param v  Value to assign to font.
   */
  
  public void setFont(Font v) {
    this.font = v;
  }

  /**
   * Returns the minimum bounding rectangle of this Graphic.
   * @return The minimum bounding rectangle of this Graphic.
   */
  public abstract Rectangle2D getMBR();

  /**
   * Scales this Graphic to the specified dimensions.
   * @param width The width required.
   * @param height The height required.
   */
  public abstract void scaleTo(double width, double height);

  /**
   * Scales this Graphic by the specified amounts.
   * @param scaleX The amount to scale in the X direction.
   * @param scaleY The amount to scale in the Y direction.
   */
  public abstract void scale(double scaleX, double scaleY);

  /**
   * Translates this Graphic to the specified location. The location will
   * become the center of the Graphic's bounding rectangle.
   * @param x The x coordinate to translate to.
   * @param y The y coordinate to translate to.
   */
  public abstract void translateTo(double x, double y);

  /**
   * Translates this Graphic by the specified distances.
   * @param x The distance to translate in the X direction.
   * @param y The distance to translate in the Y direction.
   */
  public abstract void translate(double translateX, double translateY);

  /**
   * Rotates this Graphic by the specified angle.
   * @param angle The angle to rotate
   */
  public abstract void rotate(double angle);

  /**
   * Applies the given AffineTransform to this Graphic.
   * @param trans The AffineTransform
   */
  public abstract void transform(AffineTransform trans);



}
