/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */



/**
 * Group.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: Group.java,v 1.1.1.1 1999/12/11 18:51:27 gjb Exp $
 *
 */

package org.csiro.svgv.graphic;

import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;


public class Group extends Graphic {
  
  public Group() {
    super();
  }
  
  public Group(String id) {
    super(id);
  }

  
  ArrayList elements = new ArrayList();
  
  /**
   * Get the value of elements.
   * @return Value of elements.
   */
  public ArrayList getElements() {
    return elements;
  }
  
  /**
   * Set the value of elements.
   * @param v  Value to assign to elements.
   */
  public void setElements(ArrayList v) {
    this.elements = v;
  }
  
  /**
   * add an element
   */

  public void addElement(Graphic g) {
    elements.add(g);
  }

  /**
   * Returns the minimum bounding rectangle of this Group.
   * @return The minimum bounding rectangle of this Group.
   */
  public Rectangle2D getMBR() {
    double minX = Double.MAX_VALUE;
    double minY = Double.MAX_VALUE;
    double maxX = Double.MIN_VALUE;
    double maxY = Double.MIN_VALUE;
    int size = elements.size();
    for (int i = 0; i < size; i++) {
      Graphic graphic = (Graphic) elements.get(i);
      Rectangle2D mbr = graphic.getMBR();
      if (mbr != null) {
	if (mbr.getMinX() < minX) minX = mbr.getMinX();
	if (mbr.getMinY() < minY) minY = mbr.getMinY();
	if (mbr.getMaxX() > maxX) maxX = mbr.getMaxX();
	if (mbr.getMaxY() > maxY) maxY = mbr.getMaxY();
      }
    }

    return new Rectangle2D.Double(minX, minY, maxX-minX, maxY-minY);

  }

  /**
   * Scales this Group to the specified dimensions.
   * @param width The width required.
   * @param height The height required.
   */
  public void scaleTo(double width, double height) {
    Rectangle2D mbr = getMBR();
    double currentWidth = mbr.getWidth();
    double currentHeight = mbr.getHeight();
    double scaleX = width/currentWidth;
    double scaleY = height/currentHeight;
    scale(scaleX, scaleY);
  }

  /**
   * Scales this Group by the specified amounts.
   * @param scaleX The amount to scale in the X direction.
   * @param scaleY The amount to scale in the Y direction.
   */
  public void scale(double scaleX, double scaleY) {
    int size = elements.size();
    for (int i = 0; i < size; i++) {
      Graphic graphic = (Graphic) elements.get(i);
      graphic.scale(scaleX, scaleY);
    }
  }

  /**
   * Translates this Group to the specified location. The location will
   * become the center of the Group's bounding rectangle.
   * @param x The x coordinate to translate to.
   * @param y The y coordinate to translate to.
   */
  public void translateTo(double x, double y) {
    Rectangle2D mbr = getMBR();
    double currentCenterX = mbr.getCenterX();
    double currentCenterY = mbr.getCenterY();
    double translateX = x - currentCenterX;
    double translateY = y - currentCenterY;
    translate(translateX, translateY);
  }

  /**
   * Translates this Group by the specified distances.
   * @param x The distance to translate in the X direction.
   * @param y The distance to translate in the Y direction.
   */
  public void translate(double translateX, double translateY) {
    int size = elements.size();
	for (int i = 0; i < size; i++) {
	  Graphic graphic = (Graphic) elements.get(i);
      graphic.translate(translateX, translateY);
    }
  }

  /**
   * Rotates this Graphic by the specified angle.
   * @param angle The angle to rotate
   */
  public void rotate(double angle) {
    int size = elements.size();
    for (int i = 0; i < size; i++) {
      Graphic graphic = (Graphic) elements.get(i);
      graphic.rotate(angle);
    }
  }

  /**
   * Applies the given AffineTransform to this Group.
   * @param trans The AffineTransform
   */
  public void transform(AffineTransform trans) {
    int size = elements.size();
    for (int i = 0; i < size; i++) {
      Graphic graphic = (Graphic) elements.get(i);
      graphic.transform(trans);
    }
 
  }



} // Group
