/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */



/**
 * Image.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: Image.java,v 1.1.1.1 1999/12/11 18:51:27 gjb Exp $
 *
 */

package org.csiro.svgv.graphic;

import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.Shape;

public class Image extends Graphic {

  public Image() {
    super();
  }

  public Image(String id) {
    super(id);
  }

  double x;
  
  /**
   * Get the value of x.
   * @return Value of x.
   */
  
  public double getX() {
    return x;
  }
  
  /**
   * Set the value of x.
   * @param v  Value to assign to x.
   */
  
  public void setX(double v) {
    this.x = v;
  }

  double y;
  
  /**
   * Get the value of y.
   * @return Value of y.
   */
  
  public double getY() {
    return y;
  }
  
  /**
   * Set the value of y.
   * @param v  Value to assign to y.
   */
  
  public void setY(double v) {
    this.y = v;
  }

  double width;
  
  /**
   * Get the value of width.
   * @return Value of width.
   */
  
  public double getWidth() {
    return width;
  }
  
  /**
   * Set the value of width.
   * @param v  Value to assign to width.
   */
  
  public void setWidth(double v) {
    this.width = v;
  }
  
  double height;
  
  /**
   * Get the value of height.
   * @return Value of height.
   */
  
  public double getHeight() {
    return height;
  }
  
  /**
   * Set the value of height.
   * @param v  Value to assign to height.
   */
  
  public void setHeight(double v) {
    this.height = v;
  }
  
  java.awt.Image image = null;
  
  /**
   * Get the image
   * @return the image
   */
  public java.awt.Image getImage() {
    return image;
  }

  /**
   * Set the image
   * @param v  Value to assign to image.
   */
  public void setImage(java.awt.Image v) {
    this.image = v;
  }

  Rectangle2D mbr = null;

  /**
   * Returns the minimum bounding rectangle of this Image.
   * @return The minimum bounding rectangle of this Image.
   */
  public Rectangle2D getMBR() {
    if (mbr == null) {
      Rectangle2D rect = new Rectangle2D.Double(x, y, width, height);
      Shape s = at.createTransformedShape(rect);
      mbr = s.getBounds2D();
    }

    return mbr;
  }

  
  AffineTransform at = new AffineTransform();

  /**
   * Returns the affine transform that applies to this image
   */

  public AffineTransform getTransform() {
    return at;
  }


  /**
   * Scales this Image to the specified dimensions.
   * @param width The width required.
   * @param height The height required.
   */
  public void scaleTo(double width, double height) {
    Rectangle2D mbr = getMBR();
    if (mbr != null) {
      double currentWidth = mbr.getWidth();
      double currentHeight = mbr.getHeight();
      double scaleX = width/currentWidth;
      double scaleY = height/currentHeight;
      scale(scaleX, scaleY);
    }
  }

  /**
   * Scales this Image by the specified amounts.
   * @param scaleX The amount to scale in the X direction.
   * @param scaleY The amount to scale in the Y direction.
   */
  public void scale(double scaleX, double scaleY) {
    at.scale(scaleX, scaleY);
    mbr = null;
  }

  /**
   * Translates this Image to the specified location. The location will
   * become the center of the Image's bounding rectangle.
   * @param x The x coordinate to translate to.
   * @param y The y coordinate to translate to.
   */
  public void translateTo(double x, double y) {
    Rectangle2D mbr = getMBR();
    if (mbr != null) {
      double currentCenterX = mbr.getCenterX();
      double currentCenterY = mbr.getCenterY();
      double translateX = x - currentCenterX;
      double translateY = y - currentCenterY;
      translate(translateX, translateY);
    }
  }

  /**
   * Translates this Image by the specified distances.
   * @param x The distance to translate in the X direction.
   * @param y The distance to translate in the Y direction.
   */
  public void translate(double translateX, double translateY) {
    at.translate(translateX, translateY);
    mbr = null;
  }

  /**
   * Rotates this Graphic by the specified angle.
   * @param angle The angle to rotate
   */
  public void rotate(double angle) {
    at.rotate(angle);
    mbr = null;
  }

  /**
   * Applies the given AffineTransform to this Image
   * @param trans The AffineTransform
   */
  public void transform(AffineTransform trans) {
    at.concatenate(trans);
    mbr = null;
  }

}



