/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */



/**
 * Path.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: Path.java,v 1.1.1.1 1999/12/11 18:51:28 gjb Exp $
 *
 */

package org.csiro.svgv.graphic;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;

public class Path extends Graphic {

  public Path() {
    super();
  }

  public Path(String id) {
    super(id);
  }

  
  GeneralPath generalPath = new GeneralPath();
  
  /**
   * Get the value of generalPath.
   * @return Value of generalPath.
   */
  public GeneralPath getGeneralPath() {
    return generalPath;
  }

  /**
   * Set the value of generalPath.
   * @param v  Value to assign to generalPath.
   */
  public void setGeneralPath(GeneralPath v) {
    this.generalPath = v;
    mbr = null;
  }

  /**
   * The MBR of this path
   */
  Rectangle2D mbr = null;

  /**
   * Returns the minimum bounding rectangle of this Path.
   * @return The minimum bounding rectangle of this Path.
   */
  public Rectangle2D getMBR() {
    if (mbr == null) {
      if (generalPath != null) {
        mbr = generalPath.getBounds2D();
      }
    }

    return mbr;
  }

  /**
   * Scales this Path to the specified dimensions.
   * @param width The width required.
   * @param height The height required.
   */
  public void scaleTo(double width, double height) {
    Rectangle2D mbr = getMBR();
    double currentWidth = mbr.getWidth();
    double currentHeight = mbr.getHeight();
    double scaleX = width/currentWidth;
    double scaleY = height/currentHeight;
    scale(scaleX, scaleY);
  }

  /**
   * Scales this Path by the specified amounts.
   * @param scaleX The amount to scale in the X direction.
   * @param scaleY The amount to scale in the Y direction.
   */
  public void scale(double scaleX, double scaleY) {
    transform(AffineTransform.getScaleInstance(scaleX, scaleY));
  }

  /**
   * Translates this Path to the specified location. The location will
   * become the center of the Path's bounding rectangle.
   * @param x The x coordinate to translate to.
   * @param y The y coordinate to translate to.
   */
  public void translateTo(double x, double y) {
    Rectangle2D mbr = getMBR();
    double currentCenterX = mbr.getCenterX();
    double currentCenterY = mbr.getCenterY();
    double translateX = x - currentCenterX;
    double translateY = y - currentCenterY;
    translate(translateX, translateY);
  }

  /**
   * Translates this Path by the specified distances.
   * @param translateX The distance to translate in the X direction.
   * @param translateY The distance to translate in the Y direction.
   */
  public void translate(double translateX, double translateY) {
    transform(AffineTransform.getTranslateInstance(translateX, translateY));
  }
  
  /**
   * Rotates this Graphic by the specified angle.
   * @param angle The angle to rotate
   */
  public void rotate(double angle) {
    transform(AffineTransform.getRotateInstance(angle));
  }

  /**
   * Applies the given AffineTransform to this Path.
   * @param trans The AffineTransform
   */
  public void transform(AffineTransform trans) {  
    generalPath.transform(trans);
    mbr = null;
  }
}
