/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */



/**
 * Text.java
 *
 * Created: Thu Jun  3 15:49:49 1999
 * $Id: Text.java,v 1.2 2000/05/06 21:40:17 gjb Exp $
 *
 */

package org.csiro.svgv.graphic;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
import java.awt.font.*;
import java.awt.Font;

public class Text extends Graphic {

  public Text() {
    super();
  }

  public Text(String id) {
    super(id);
  }

  double x;
  
  /**
   * Get the value of x.
   * @return Value of x.
   */
  
  public double getX() {
    return x;
  }
  
  /**
   * Set the value of x.
   * @param v  Value to assign to x.
   */
  
  public void setX(double v) {
    this.x = v;
  }

  double y;
  
  /**
   * Get the value of y.
   * @return Value of y.
   */
  
  public double getY() {
    return y;
  }
  
  /**
   * Set the value of y.
   * @param v  Value to assign to y.
   */
  
  public void setY(double v) {
    this.y = v;
  }

  private TextLayout tl;

  public TextLayout getLayout() {
    return this.tl;
  }

  GeneralPath shape = null;
  
  /**
   * Get the value of shape.
   * @return Value of shape.
   */
  public Shape getShape() {
    return shape;
  }

  /**
   * Set the value of shape.
   * @param v  Value to assign to shape.
   */
  private void setShape(GeneralPath v) {
    this.shape = v;
    mbr = null;
  }

  Rectangle2D mbr = null;

  /**
   * Returns the minimum bounding rectangle of this Text.
   * @return The minimum bounding rectangle of this Text.
   */
  public Rectangle2D getMBR() {
    if (mbr == null) {
      if (shape != null) {
        mbr = shape.getBounds2D();
      }
    }
    return mbr;
  }

  String string = null;
  
  /**
   * Get the value of string.
   * @return Value of string.
   */
  
  public String getString() {
    return string;
  }
  
  /**
   * Set the value of string.
   * @param v  Value to assign to string.
   */
  
  public void setString(String v) {
    this.string = v;
    shape = null;
    mbr = null;
  }

  /**
   * Scales this Text to the specified dimensions.
   * @param width The width required.
   * @param height The height required.
   */
  public void scaleTo(double width, double height) {
    Rectangle2D mbr = getMBR();
    if (mbr != null) {
      double currentWidth = mbr.getWidth();
      double currentHeight = mbr.getHeight();
      double scaleX = width/currentWidth;
      double scaleY = height/currentHeight;
      scale(scaleX, scaleY);
    }
  }

  /**
   * Scales this Text by the specified amounts.
   * @param scaleX The amount to scale in the X direction.
   * @param scaleY The amount to scale in the Y direction.
   */
  public void scale(double scaleX, double scaleY) {
    transform(AffineTransform.getScaleInstance(scaleX, scaleY));
  }

  /**
   * Translates this Text to the specified location. The location will
   * become the center of the Text's bounding rectangle.
   * @param x The x coordinate to translate to.
   * @param y The y coordinate to translate to.
   */
  public void translateTo(double x, double y) {
    Rectangle2D mbr = getMBR();
    if (mbr != null) {
      double currentCenterX = mbr.getCenterX();
      double currentCenterY = mbr.getCenterY();
      double translateX = x - currentCenterX;
      double translateY = y - currentCenterY;
      translate(translateX, translateY);
    }
  }

  /**
   * Translates this Text by the specified distances.
   * @param x The distance to translate in the X direction.
   * @param y The distance to translate in the Y direction.
   */
  public void translate(double translateX, double translateY) {
    transform(AffineTransform.getTranslateInstance(translateX, translateY));
  }

  /**
   * Rotates this Graphic by the specified angle.
   * @param angle The angle to rotate
   */
  public void rotate(double angle) {
    transform(AffineTransform.getRotateInstance(angle));
  }

  /**
   * Applies the given AffineTransform to this Text
   * @param trans The AffineTransform
   */
  public void transform(AffineTransform trans) {
     if (shape != null)
        shape.transform(trans);
     mbr = null;
  }

  public void createShape() {
    if (string != null && font != null) {
      tl = new TextLayout(string, font, new FontRenderContext(null,true,true));
      Shape textShape = tl.getOutline(null);
      GeneralPath path = new GeneralPath();
      path.append(textShape,false);
      shape = path;
    }

  }

}




