/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * DescElement.java
 *
 * $Id: DescElement.java,v 1.1.1.1 1999/12/11 18:51:28 gjb Exp $
 *
 */

package org.csiro.svgv.reader;


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * The DescElement class describes an SVG desc element.
 */
public class DescElement implements XmlElement {

    private String _descText = null;

    private static final String _content = "structured text";

    /**
     * Constructs a new DescElement.
     */
    public DescElement() {
    }

    /**
     * Constructs and initializes a DescElement with the given description text.
     * @param descText The value to use when initializing the description text.
     */
    public DescElement(String descText) {
        _descText = descText;
    }

    /**
     * Sets the description text to the specified value.
     * @param descText The value to use when setting the description text.
     */
    public void setDescriptionText(String descText) {
        _descText = descText;
    }

    /**
     * Returns the description text of this element.
     * @return The description text of this element.
     */
    public String getDescriptionText() {
        return _descText;
    }


    // XmlElement stuff

    /**
     * Returns the tag that identifies a desc element in an SVG/XML file.
     * @return The tag that identifies a desc element in an SVG/XML file.
     */
    public String getTag() {
        return "desc";
    }

    /**
     * Reads in DescElement data from the given XML Element.
     * @param elem The XML Element that contains data to be read.
     */
    public void read(Element elem) {
        // read text for description
        if (elem.getFirstChild() != null) {
            _descText = elem.getFirstChild().getNodeValue();
        }
    }

    /**
     * Inserts data into the given Document.
     * The data will become a child of the given parent element.
     *
     * @param doc The Document to write to.
     * @param parent Indicates the parent element of the data to be written.
     */
    public void write(Document doc, Node parent) {

        // create new element for this desc element
        Element desc = doc.createElement(getTag());

        // set text for desc
        if (_descText != null)
            desc.appendChild(doc.createTextNode(_descText));

        // append new element to the parent
        parent.appendChild(desc);
    }
} 