/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * SvgDocument.java
 *
 * $Id: SvgDocument.java,v 1.11 2000/05/08 03:12:39 will Exp $
 *
 */

package org.csiro.svgv.reader;

import java.io.*;
import java.net.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.csiro.svgv.graphic.*;
import EDU.Washington.grad.gjb.cassowary.*;

/**
 * The SvgDocument class is resposible for reading and writing whole SVG
 * documents.
 */
public class SvgDocument implements ErrorHandler {

  /**
   * The tag of the root element in an SVG document.
   */
  public static final String rootTag = "svg";

  public static String svgFilePath = "";

  /**
   * A pointer to the root svg element in the current document.
   */
  private SvgElement _rootSvgElement = null;

  /**
   * Constructs a new SvgDocument.
   */   

  public java.util.Hashtable m_variable_name_object_hash;
  public ClSimplexSolver m_simplex_solver;

  public SvgDocument() {
    m_variable_name_object_hash = new java.util.Hashtable();
    m_simplex_solver = new ClSimplexSolver();

    m_constraints = new java.util.Vector();
  }

  private java.util.Vector m_constraints;

  public void addConstraint(ClConstraint constraint)
  {
    m_constraints.add(constraint);
  }

  public void addConstraintsToSolver()
  {
    long start_time = java.lang.System.currentTimeMillis();

    java.util.Enumeration constraint_iterator = m_constraints.elements();

    m_simplex_solver.setAutosolve(false);

    while (constraint_iterator.hasMoreElements())
    {
      ClConstraint constraint = (ClConstraint) constraint_iterator.nextElement();

      try
      {
        m_simplex_solver.addConstraint(constraint);
      }
      catch (Exception exception)
      {
        System.err.println("Exception in adding constraint");
        exception.printStackTrace();
      }
    }

    m_simplex_solver.setAutosolve(true);

    long end_time = java.lang.System.currentTimeMillis();

    long total_time = end_time - start_time;

    System.out.println("It took " + total_time + " milliseconds to add the constraints.");
  }


  /**
   * Returns the Graphic object that represents this SVG document.
   * @return The Graphic object that represents this SVG document.
   */
  public Graphic getSvgGraphics() {
    if (_rootSvgElement != null)
      return _rootSvgElement.getGraphic();
    return new Group();
  }

  public void outputCnVarValues(PrintStream o) {
    if (m_variable_name_object_hash == null)
      return;

    o.println("Constraint variables:");
    java.util.Set entries = m_variable_name_object_hash.entrySet();
    for (java.util.Iterator it = entries.iterator(); it.hasNext(); ) {
      java.util.Map.Entry e = (java.util.Map.Entry) it.next();
      double value = ((ClVariable) m_variable_name_object_hash.get(e.getKey())).value();
      o.println(e.getKey().toString() + " = " + value);
    }

  }

  public String handlePossibleVariable(String possible_variable)
  {
    String return_value;
    
    // check to see if possible_variable begins with a dollar sign and strip it
    
    possible_variable.trim();
    
    if (possible_variable.startsWith("$"))
      {
        possible_variable = possible_variable.substring(1, possible_variable.length());
      }
    
    // check to see if it is a variable name
    // if not, look it up in hash
    
    if (m_variable_name_object_hash == null)
      {
        System.err.println("Svg Document doesn't have a hash table");
      }
    if (m_variable_name_object_hash.containsKey(possible_variable))
      {
        return_value = new Double( ((ClVariable) m_variable_name_object_hash.get(possible_variable)).value()).toString();
        // System.err.println("	Returning <" + possible_variable + "> as " + return_value);
      }
    else
      {
        return_value = possible_variable;
      }
    
    return return_value;
  }
  
  /**
   * Returns the width of the SVG graphics to be displayed.
   * @return The width (in pixels) of the SVG graphics to be displayed.
   */
  public int getSvgWidth() {
    if (_rootSvgElement != null)
      return _rootSvgElement.getWidth();
    return 600;
  }

  /**
   * Returns the height of the SVG graphics to be displayed.
   * @return The height (in pixels) of the SVG graphics to be displayed.
   */
  public int getSvgHeight() {
    if (_rootSvgElement != null)
      return _rootSvgElement.getHeight();
    return 600;
  }

  /**
   * Returns the title of this SVG document.
   * @return The title of this SVG document or null if there is no title.
   */
  public String getSvgTitle() {
    if (_rootSvgElement != null && _rootSvgElement.getTitle() != null)
      return _rootSvgElement.getTitle().getTitleText();
    return null;
  }

  /**
   * Reads the SVG file from the given url and creates and sets the appropriate
   * data structures. If the urlString does not start with and appropriate
   * protocol descriptor (eg. "http://" or "ftp://") then it is assumed to
   * be a local file.
   * @param urlString Url of the SVG file to be read.
   * @return True if the url could be read, false otherwise.
   */

  public boolean readSvgUrl(String urlString) {

    com.ibm.xml.parsers.DOMParser parser = new com.ibm.xml.parsers.DOMParser();
    parser.setErrorHandler(this);
    try {
      parser.parse(urlString);
    } catch (SAXException e) {
      System.err.println("SAXException in parsing XML");
      e.printStackTrace();
      return false;
    } catch (IOException e) {
      System.err.println("IOException in parsing XML");
      e.printStackTrace();
      return false;
    }

    Document doc = parser.getDocument();
    Element root = doc.getDocumentElement();
    root.normalize();
    _rootSvgElement = new SvgElement(this);
    _rootSvgElement.read(root);

    if ((urlString.indexOf('/') != -1) || (urlString.indexOf('\\') != -1)) {
      int index = urlString.lastIndexOf('/');
      if (index == -1)
	index = urlString.lastIndexOf('\\');
      svgFilePath = urlString.substring(0,index+1);
    }

    addConstraintsToSolver();

    return true;

  }
  

  // Commented out these things when changing to IBM XML Parser
  // Have to clean this up



  //    /**
  //     * Reads the specified SVG file and creates and sets the appropriate
  //     * data structures.
  //     * @param filename Name of the SVG file to be read.
  //     * @return True if the file could be read, false otherwise
  //     */
  //    public boolean readSvgFile(String filename) {
  //      try {
  //        if ((filename.indexOf('/') != -1) || (filename.indexOf('\\') != -1)) {
  //  	int index = filename.lastIndexOf('/');
  //  	if (index == -1)
  //  	  index = filename.lastIndexOf('\\');
  //  	svgFilePath = filename.substring(0,index+1);
  //        }
  //        InputSource input = Resolver.createInputSource(new File(filename));
  //        Document doc = openXmlFile(input);
  //        Element root = doc.getDocumentElement();
  //        root.normalize();
  //        _rootSvgElement = new SvgElement();
  //        _rootSvgElement.read(root);
  //      } catch (FileNotFoundException e) {
  //        System.out.println("File not found: " + filename);
  //        return false;
  //      } catch (IOException e) {
  //        System.out.println("Could not create XML input source from file: " + filename);
  //        return false;
  //      }

  //      return true;
  //    }

  //    /**
  //     * Reads in SVG data from the specified file and constructs the
  //     * appropriate data structure.
  //     * @param in The input stream to read from.
  //     * @return True if read worked, false otherwise.
  //     */
  //    public boolean readSvgFile(InputStream in) {
  //      try {
  //        com.ibm.xml.parsers.DOMParser parser = new com.ibm.xml.parsers.DOMParser();
  //        parser.setErrorHandler(this);

  //        InputSource input = Resolver.createInputSource("text/xml;charset=utf-8", in, true, null);
  //        Document doc = openXmlFile(input);
  //        Element root = doc.getDocumentElement();
  //        root.normalize();
  //        _rootSvgElement = new SvgElement();
  //        _rootSvgElement.read(root);
  //      } catch (IOException e) {
  //        System.out.println("Could not create XML input source from given input stream");
  //        return false;
  //      }
  //      return true;
  //    }

  //    /**
  //     * Creates a new Document by reading XML data from the given input
  //     * stream.
  //     * @param in The input stream to read from.
  //     * @return Document that represents the data in the XML file
  //     */
  //    private Document openXmlFile(InputSource input) {
  //      try {
  //        Document doc = Document.createXmlDocument (input, true);
  //        return doc;

  //      } catch (IOException e) {
  //        System.out.println("IOException in openXmlFile");
  //        return createDefaultSvgDocument();

  //      } catch (SAXParseException err) {
  //        System.out.println ("** Parsing error"
  //  			  + ", line " + err.getLineNumber ()
  //  			  + ", uri " + err.getSystemId ());
  //        System.out.println("   " + err.getMessage ());
  //        return createDefaultSvgDocument();

  //      } catch (SAXException e) {
  //        Exception x = e.getException ();
  //        ((x == null) ? e : x).printStackTrace ();
  //        return createDefaultSvgDocument();
  //      }
  //    }

  private Document createDefaultSvgDocument() {
    Document doc = new com.ibm.xml.dom.DocumentImpl();
    Element root = doc.createElement(rootTag);
    root.setAttribute("width", "600");
    root.setAttribute("height", "600");
    doc.appendChild(root);
    return doc;
  }

  /**
   * Writes the SVG data to the given output stream.
   * @param out The output stream to write to
   */
  public void writeSvgFile(OutputStream out) {

    // create a new XmlDocuemnt
    Document doc = new com.ibm.xml.dom.DocumentImpl();

    // set document type definition
    //doc.setDoctype("-//W3C//DTD SVG June 1999//EN" ,
    //		   "http://www.w3.org/Graphics/SVG/svg-19990812.dtd", null);

    // create an empty svg element
    if (_rootSvgElement == null)
      _rootSvgElement = new SvgElement(this);

    // fill Document with data
    _rootSvgElement.write(doc, doc);

    // temporarily disabled while changing to IBM XML Parser

    // write it to the stream
    //      try {
    //        doc.write(out);
    //      } catch (IOException e) {
    //        System.out.println("Failed to write SVG to output stream ");
    //      }
  }

  /**
   * Writes the SVG data to the given file.
   * @param filename Name of SVG file to be written.
   */
  public void writeSvgFile(String filename) {

    // write it to file
    try {
      FileOutputStream out = new FileOutputStream (filename);
      writeSvgFile(out);
      out.close();
    } catch (IOException e) {
      System.err.println("Failed to write SVG file: " + filename);
    }
  }

  /** Warning. */
  public void warning(SAXParseException ex) {
    System.err.println("[Warning] "+
		       getLocationString(ex)+": "+
		       ex.getMessage());
  }
  
  /** Error. */
  public void error(SAXParseException ex) {
    System.err.println("[Error] "+
		       getLocationString(ex)+": "+
		       ex.getMessage());
  }
  
  /** Fatal error. */
  public void fatalError(SAXParseException ex) throws SAXException {
    System.err.println("[Fatal Error] "+
		       getLocationString(ex)+": "+
		       ex.getMessage());
    throw ex;
  }

  /** Returns a string of the location. */
  private String getLocationString(SAXParseException ex) {
    StringBuffer str = new StringBuffer();
    
    String systemId = ex.getSystemId();
    if (systemId != null) {
      int index = systemId.lastIndexOf('/');
      if (index != -1) 
	systemId = systemId.substring(index + 1);
      str.append(systemId);
    }
    str.append(':');
    str.append(ex.getLineNumber());
    str.append(':');
    str.append(ex.getColumnNumber());

    return str.toString();
  }

} 
