/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * SvgStdGraphic.java
 *
 * $Id: SvgStdGraphic.java,v 1.1.1.1 1999/12/11 18:51:28 gjb Exp $
 *
 */

package org.csiro.svgv.reader;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.csiro.svgv.graphic.*;
import java.util.StringTokenizer;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.BasicStroke;
import java.awt.Color;
import java.util.Vector;
import java.awt.geom.AffineTransform;

/**
 * The SvgStdGraphic calss is used as a base class for all SVG elements that
 * have the std_g_attrs as defined in the DTD. Simply provides functions that
 * read, write, set and get these attribtues.
 */
public abstract class SvgStdGraphic extends SvgGraphic {

    // transform attribute
    protected String _transform = null;

    // system_required
    protected String _system_required = null;

    // g_event_handlers
    protected String _onmousedown = null;
    protected String _onmouseup = null;
    protected String _onclick = null;
    protected String _ondblclick = null;
    protected String _onmouseover = null;
    protected String _onmousemove = null;
    protected String _onmouseout = null;
    protected String _onkeydown = null;
    protected String _onkeypress = null;
    protected String _onkeyup = null;
    protected String _onload = null;
    protected String _onselect = null;

    public String getTransform() {
        return _transform;
    }

    public String getOnMouseDown() {
        return _onmousedown;
    }
    public String getOnMouseUp() {
        return _onmouseup;
    }
    public String getOnClick() {
        return _onclick;
    }
    public String getOnDblClick() {
        return _ondblclick;
    }
    public String getOnMouseOver() {
        return _onmouseover;
    }
    public String getOnMouseMove() {
        return _onmousemove;
    }
    public String getOnMouseOut() {
        return _onmouseout;
    }
    public String getOnKeyDown() {
        return _onkeydown;
    }
    public String getOnKeyPress() {
        return _onkeypress;
    }
    public String getOnKeyUp() {
        return _onkeyup;
    }
    public String getOnLoad() {
        return _onload;
    }
    public String getOnSelect() {
        return _onselect;
    }

    public void setTransform(String transform) {
        _transform = transform;
    }
    public void setSystemRequired(String systemRequired) {
        _system_required = systemRequired;
    }
    public void setOnMouseDown(String onMouseDown) {
        _onmousedown = onMouseDown;
    }
    public void setOnMouseUp(String onMouseUp) {
        _onmouseup = onMouseUp;
    }
    public void setOnClick(String onClick) {
        _onclick = onClick;
    }
    public void setOnDblClick(String onDblClick) {
        _ondblclick = onDblClick;
    }
    public void setOnMouseOver(String onMouseOver) {
        _onmouseover = onMouseOver;
    }
    public void setOnMouseMove(String onMouseMove) {
        _onmousemove = onMouseMove;
    }
    public void setOnMouseOut(String onMouseOut) {
        _onmouseout = onMouseOut;
    }
    public void setOnKeyDown(String onKeyDown) {
        _onkeydown = onKeyDown;
    }
    public void setOnKeyPress(String onKeyPress) {
        _onkeypress = onKeyPress;
    }
    public void setOnKeyUp(String onKeyUp) {
        _onkeyup = onKeyUp;
    }
    public void setOnLoad(String onLoad) {
        _onload = onLoad;
    }
    public void setOnSelect(String onSelect) {
        _onselect = onSelect;
    }


    /**
     * Reads the standard graphic attributes from the given XML Element.
     * @param elem The XML Element to read from.
     */
    public void readStandardGraphicAttributes(Element elem) {

        String att;

        // read id attribute
        att = elem.getAttribute("id");
        if (att.length() > 0)
            _id = att;

        att = elem.getAttribute("xml:lang");
        if (att.length() > 0)
            _xmlLang = att;

        att = elem.getAttribute("xml:space");
        if (att.length() > 0)
            _xmlSpace = att;

        // read class attriubte
        att = elem.getAttribute("class");
        if (att.length() > 0)
            _class = att;

        // read style attribute
        att = elem.getAttribute("style");
        if (att.length() > 0)
            _style = att;

        // read transform attribute
        att = elem.getAttribute("transform");
        if (att.length() > 0)
            _transform = att;

        // read system-required attribute
        att = elem.getAttribute("system-required");
        if (att.length() > 0)
            _system_required = att;

        // read g_eventhandlers

        att = elem.getAttribute("onmousedown");
        if (att.length() > 0)
            _onmousedown = att;

        att = elem.getAttribute("onmouseup");
        if (att.length() > 0) {
            _onmouseup = att;
        }
        att = elem.getAttribute("onclick");
        if (att.length() > 0)
            _onclick = att;

        att = elem.getAttribute("ondblclick");
        if (att.length() > 0)
            _ondblclick = att;

        att = elem.getAttribute("onmouseover");
        if (att.length() > 0)
            _onmouseover = att;

        att = elem.getAttribute("onmousemove");
        if (att.length() > 0)
            _onmousemove = att;

        att = elem.getAttribute("onmouseout");
        if (att.length() > 0)
            _onmouseout = att;

        att = elem.getAttribute("onkeydown");
        if (att.length() > 0)
            _onkeydown = att;

        att = elem.getAttribute("onkeypress");
        if (att.length() > 0)
            _onkeypress = att;

        att = elem.getAttribute("onkeyup");
        if (att.length() > 0)
            _onkeyup = att;

        att = elem.getAttribute("onload");
        if (att.length() > 0)
            _onload = att;

        att = elem.getAttribute("onmouseup");
        if (att.length() > 0)
            _onselect = att;
    }

    /**
     * Sets the standard graphic attributes of an XML Element.
     * @param elem The XML element that needs its attributes set.
     */
    public void setStandardGraphicAttributes(Element elem) {
        if (_id != null)
            elem.setAttribute("id", _id);
        if (_xmlLang != null)
            elem.setAttribute("xml:lang", _xmlLang);
        if (_xmlSpace != null)
            elem.setAttribute("xml:space", _xmlSpace);
        if (_class != null)
            elem.setAttribute("class", _class);
        if (_style != null)
            elem.setAttribute("style", _style);
        if (_transform != null)
            elem.setAttribute("transform", _transform);
        if (_system_required != null)
            elem.setAttribute("system-required", _system_required);
        if (_onmousedown != null)
            elem.setAttribute("onmousedown", _onmousedown);
        if (_onmouseup != null)
            elem.setAttribute("onmouseup", _onmouseup);
        if (_onclick != null)
            elem.setAttribute("onclick", _onclick);
        if (_ondblclick != null)
            elem.setAttribute("ondblclick", _ondblclick);
        if (_onmouseover != null)
            elem.setAttribute("onmouseover", _onmouseover);
        if (_onmousemove != null)
            elem.setAttribute("onmousemove", _onmousemove);
        if (_onmouseout != null)
            elem.setAttribute("onmouseout", _onmouseout);
        if (_onkeydown != null)
            elem.setAttribute("onkeydown", _onkeydown);
        if (_onkeypress != null)
            elem.setAttribute("onkeypress", _onkeypress);
        if (_onkeyup != null)
            elem.setAttribute("onkeyup", _onkeyup);
        if (_onload != null)
            elem.setAttribute("onload", _onload);
        if (_onselect != null)
            elem.setAttribute("onselect", _onselect);
     }



}