/*
 * Copyright (C) 1999
 * CSIRO Mathematical and Information Sciences Australia
 * Spatial Information Systems Project
 *
 * http://www.cmis.csiro.au/svg
 */


/**
 * XmlElement.java
 *
 * $Id: XmlElement.java,v 1.1.1.1 1999/12/11 18:51:28 gjb Exp $
 *
 */

package org.csiro.svgv.reader;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * A base class for all elements that may appear in an SVG/XML file.
 */
public interface XmlElement {

    /**
     * Returns the tag that identifies an SVG element in an SVG/XML file.
     * @return The tag that identifies an SVG element in an SVG/XML file.
     */
    public String getTag();

    /**
     * Reads in SVG data from given XML Element.
     *
     * @param elem The XML Element that contains data to be read.
     */
    public void read(Element elem);

    /**
     * Inserts SVG element data into the given Document at the specified position.
     * The SVG element data will become a child of the given parent node.
     *
     * @param doc The Document to write to.
     * @param parent The parent node of the SVG element data to be written.
     */
    public void write(Document doc, Node parent);

}
